import argparse
import os, sys, time
import networkx as nx
import math
import ass_kissat_impl as colmip


def read_dimacs(filename):
    print("Reading graph from", filename)
    G = nx.Graph()
    with open(filename, 'r') as fp:
        line = fp.readline()

        while line:
            tokens = line.split()
            if len(tokens) == 0 :
                pass
            elif tokens[0] == "p":
                nnodes = int(tokens[2])
                nedges = int(tokens[3])
                G.add_nodes_from(list(range(1,nnodes+1)))
            elif tokens[0] == "e":
               G.add_edge(int(tokens[1]), int(tokens[2]))
            line = fp.readline()

    print ("G has", G.number_of_nodes(), " nodes and ", G.number_of_edges(), " edges")
    if (nnodes != G.number_of_nodes()):
        print("nnodes != G.number_of_nodes() (", nnodes, "!=", G.number_of_nodes(), ")")
        print (G.nodes())
        sys.exit("Inconsistent number of nodes.")
    if (nedges != G.number_of_edges()):
        print("nedges != G.number_of_edges() (", nedges, "!=", G.number_of_edges(), ")")
        sys.exit("Inconsistent number of edges.")

    return G

def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--dimacs_graph",  default=[], required=True)
    parser.add_argument("--solution_file",  default="color.sol", required=False)
    parser.add_argument("--time_limit",  default="60", required=False)
    return parser.parse_args()



def _main():
    args = get_args()
    filename = args.dimacs_graph
    print ("filename", filename)
    G = read_dimacs(filename)
    PRINT = 1 + 2  # Gurobi output + layout chanel


    sol = colmip.color(G, PRINT, int(args.time_limit), args.solution_file)
    print (sol)
    lb = sol[6]
    ub = sol[7]
    print ("Coloring took ", sol[8], "seconds.")
    if (lb == ub):
        print("The coloring number is", int(ub),".")
    else:
        print("The coloring number is between ", math.ceil(lb),"and", int(ub),".")


if __name__ == "__main__":  # don't run if this script is imported as module
    _main()
